/* Copyright (C) 2007 - 2008 MySQL AB, 2008 Sun Microsystems, Inc.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

There are special exceptions to the terms and conditions of the GPL
as it is applied to this software. View the full text of the
exception in file EXCEPTIONS-CONNECTOR-C++ in the directory of this
software distribution.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "resources.h"
#include <fstream>

namespace testsuite
{
  namespace resources
  {
    void CharsetMapping::Init()
    {
      STATIC_CHARSET_TO_NUM_BYTES_MAP.insert(
        std::map<String, unsigned int>::value_type(_T("big5"), 2U ));
    
      STATIC_CHARSET_TO_NUM_BYTES_MAP.insert(
        std::map<String, unsigned int>::value_type(_T("dec8" ), 1U ));
      STATIC_CHARSET_TO_NUM_BYTES_MAP.insert(
        std::map<String, unsigned int>::value_type(_T("cp850"), 1U ));
      STATIC_CHARSET_TO_NUM_BYTES_MAP.insert(
        std::map<String, unsigned int>::value_type(_T("hp8"), 1U ));
      STATIC_CHARSET_TO_NUM_BYTES_MAP.insert(
        std::map<String, unsigned int>::value_type(_T("koi8r"), 1U ));
      STATIC_CHARSET_TO_NUM_BYTES_MAP.insert(
        std::map<String, unsigned int>::value_type(_T("latin1"), 1U ));
      STATIC_CHARSET_TO_NUM_BYTES_MAP.insert(
        std::map<String, unsigned int>::value_type(_T("latin2"), 1U ));
      STATIC_CHARSET_TO_NUM_BYTES_MAP.insert(
        std::map<String, unsigned int>::value_type(_T("swe7"), 1U ));
      STATIC_CHARSET_TO_NUM_BYTES_MAP.insert(
        std::map<String, unsigned int>::value_type(_T("ascii"), 1U ));
      STATIC_CHARSET_TO_NUM_BYTES_MAP.insert(
        std::map<String, unsigned int>::value_type(_T("ujis"), 3U ));
      STATIC_CHARSET_TO_NUM_BYTES_MAP.insert(
        std::map<String, unsigned int>::value_type(_T("sjis"), 2U ));
      STATIC_CHARSET_TO_NUM_BYTES_MAP.insert(
        std::map<String, unsigned int>::value_type(_T("hebrew"), 1U ));
      STATIC_CHARSET_TO_NUM_BYTES_MAP.insert(
        std::map<String, unsigned int>::value_type(_T("tis620"), 1U ));
      STATIC_CHARSET_TO_NUM_BYTES_MAP.insert(
        std::map<String, unsigned int>::value_type(_T("euckr"), 2U ));
      STATIC_CHARSET_TO_NUM_BYTES_MAP.insert(
        std::map<String, unsigned int>::value_type(_T("koi8u"), 1U ));
      STATIC_CHARSET_TO_NUM_BYTES_MAP.insert(
        std::map<String, unsigned int>::value_type(_T("gb2312"), 2U ));
      STATIC_CHARSET_TO_NUM_BYTES_MAP.insert(
        std::map<String, unsigned int>::value_type(_T("greek"), 1U ));
      STATIC_CHARSET_TO_NUM_BYTES_MAP.insert(
        std::map<String, unsigned int>::value_type(_T("cp1250"), 1U ));
      STATIC_CHARSET_TO_NUM_BYTES_MAP.insert(
        std::map<String, unsigned int>::value_type(_T("gbk"), 2U ));
      STATIC_CHARSET_TO_NUM_BYTES_MAP.insert(
        std::map<String, unsigned int>::value_type(_T("latin5"), 1U ));
      STATIC_CHARSET_TO_NUM_BYTES_MAP.insert(
        std::map<String, unsigned int>::value_type(_T("armscii8"), 1U ));
      STATIC_CHARSET_TO_NUM_BYTES_MAP.insert(
        std::map<String, unsigned int>::value_type(_T("utf8"), 3U ));
      STATIC_CHARSET_TO_NUM_BYTES_MAP.insert(
        std::map<String, unsigned int>::value_type(_T("ucs2"), 2U ));
      STATIC_CHARSET_TO_NUM_BYTES_MAP.insert(
        std::map<String, unsigned int>::value_type(_T("cp866"), 1U ));
      STATIC_CHARSET_TO_NUM_BYTES_MAP.insert(
        std::map<String, unsigned int>::value_type(_T("keybcs2"), 1U ));
      STATIC_CHARSET_TO_NUM_BYTES_MAP.insert(
        std::map<String, unsigned int>::value_type(_T("macce"), 1U ));
      STATIC_CHARSET_TO_NUM_BYTES_MAP.insert(
        std::map<String, unsigned int>::value_type(_T("macroman"), 1U ));
      STATIC_CHARSET_TO_NUM_BYTES_MAP.insert(
        std::map<String, unsigned int>::value_type(_T("cp852" ), 1U ));
      STATIC_CHARSET_TO_NUM_BYTES_MAP.insert(
        std::map<String, unsigned int>::value_type(_T("latin7"), 1U ));
      STATIC_CHARSET_TO_NUM_BYTES_MAP.insert(
        std::map<String, unsigned int>::value_type(_T("cp1251"), 1U ));
      STATIC_CHARSET_TO_NUM_BYTES_MAP.insert(
        std::map<String, unsigned int>::value_type(_T("cp1256" ), 1U ));
      STATIC_CHARSET_TO_NUM_BYTES_MAP.insert(
        std::map<String, unsigned int>::value_type(_T("cp1257"), 1U ));
      STATIC_CHARSET_TO_NUM_BYTES_MAP.insert(
        std::map<String, unsigned int>::value_type(_T("binary"), 1U ));
      STATIC_CHARSET_TO_NUM_BYTES_MAP.insert(
        std::map<String, unsigned int>::value_type(_T("geostd8"), 1U ));
      STATIC_CHARSET_TO_NUM_BYTES_MAP.insert(
        std::map<String, unsigned int>::value_type(_T("cp932"), 2U ));
      STATIC_CHARSET_TO_NUM_BYTES_MAP.insert(
        std::map<String, unsigned int>::value_type(_T("eucjpms"), 3U ));
    }
    
    const CharsetMapping &     CharsetMapping::Instance()
    {
      static CharsetMapping instance;
      return instance;
    }

    int LoadProperties( const String & fileName, Properties &props )
    {
      int counter= 0;

      std::ifstream propsFile( fileName.c_str() );

      if (propsFile.is_open())
      {
        String line;
        while (! propsFile.eof() )
        {
          getline (propsFile,line);

          StringUtils::trim( line );

          // Not empty line or a comment
          if (line.size() > 0 && line.c_str()[0] != '#')
          {
            String::size_type pos = line.find_first_of( "=" );

            if (pos != String::npos && pos > 0)
            {
              String key= StringUtils::trim( line.substr( 0, pos ) );
              String val= StringUtils::trim( line.substr( pos + 1 ) );

              props.insert( Properties::value_type( key, val ) );
              ++counter;
            }

          }
        }

        propsFile.close();
      }

      else
      {
        std::cout << "Unable to open file" << std::endl; 
        return -1;
      }

      return counter;
    }
  }
}
